searchNodes=[{"doc":"Utility for pulling the function call traces into your IEx shell for further analysis and experimentation. The code is built on top of Rexbug ( https://github.com/nietaki/rexbug ). Motivation: Rexbug provides a convenient way of tracing function calls by printing the trace messages to IEx shell and/or the external file. In addition, Replbug allows to materialize traces as a variable, and then analyze the call data in IEx for debugging, experimentation, collecting stats etc. Example 1: Trace DateTime.utc_now/0. Let the tracer run for 60 secs or until 10 trace messages are emitted. iex(1)&gt; Replbug.start(&quot;DateTime.utc_now/0&quot;, time: 60_000, msgs: 10) You could either wait for the system to make DateTime.utc_now/0 calls, or call it manually in IEx. At any time, stop collecting traces and get the call trace records: iex(3)&gt; call_traces = Replbug.stop %{ #PID&lt;0.1392.0&gt; =&gt; [ %{ args : [ ] , call_timestamp : ~T[13:58:20.183042] , caller_pid : # PID &lt; 0 . 1392 . 0 &gt; , function : :utc_now , module : DateTime , parent : Verk.ScheduleManager , return : ~U[2022-04-11 17:58:20.183039Z] , return_timestamp : ~T[13:58:20.183054] } , %{ args : [ ] , call_timestamp : ~T[13:58:20.184928] , caller_pid : # PID &lt; 0 . 1392 . 0 &gt; , function : :utc_now , module : DateTime , parent : Verk.ScheduleManager , return : ~U[2022-04-11 17:58:20.184925Z] , return_timestamp : ~T[13:58:20.184937] } ], #PID&lt;0.1498.0&gt; =&gt; [ %{ args : [ ] , call_timestamp : ~T[13:58:18.183879] , caller_pid : # PID &lt; 0 . 1498 . 0 &gt; , function : :utc_now , module : DateTime , parent : % Rexbug.Printing.MFA { a : 1 , f : :init , m : Oban.Plugins.Stager } , return : ~U[2022-04-11 17:58:18.183883Z] , return_timestamp : ~T[13:58:18.183899] } , %{ args : [ ] , call_timestamp : ~T[13:58:19.197818] , caller_pid : # PID &lt; 0 . 1498 . 0 &gt; , function : :utc_now , module : DateTime , parent : % Rexbug.Printing.MFA { a : 1 , f : :init , m : Oban.Plugins.Stager } , return : ~U[2022-04-11 17:58:19.197821Z] , return_timestamp : ~T[13:58:19.197836] } ], #PID&lt;0.1538.0&gt; =&gt; [ %{ args : [ ] , call_timestamp : ~T[13:58:17.925199] , caller_pid : # PID &lt; 0 . 1538 . 0 &gt; , function : :utc_now , module : DateTime , parent : % Rexbug.Printing.MFA { a : 1 , f : :init , m : Oban.Queue.Producer } , return : ~U[2022-04-11 17:58:17.925201Z] , return_timestamp : ~T[13:58:17.925216] } ] } This tells us that calls to DateTime.utc_now/0 were made by 3 processes: &lt;0.1392.0&gt; (Verk.ScheduleManager, 2 times), &lt;0.1498.0&gt; (Oban.Plugins.Stager, 2 times) and &lt;0.1538.0&gt; (Oban.Queue.Producer, once) Example 2: (get the call durations for the given MFA): iex(1)&gt; Replbug.start(&quot;DateTime&quot;, &quot;String.contains?/2&quot;], time: 60_000, msgs: 100) ...do some function calls with DateTime and/or String.contains? iex(2)&gt; Enum.each(1..4, fn _ -&gt; DateTime.utc_now() end) ....... iex(3)&gt; String.contains?(&quot;aaa&quot;, &quot;b&quot;) retrieve the call traces iex(4)&gt; calls = Replbug.stop |&gt; Replbug.calls() Get the list of calls: iex(5)&gt; Map.keys(calls) [ {DateTime, : info , 1}, {DateTime, :convert, 2}, {DateTime, :from_unix, 3}, {DateTime, :from_unix!, 3}, {DateTime, :utc_now, 0}, {DateTime, :utc_now, 1}, {String, :contains?, 2} ] Get the call durations for DataTime.utc_now/0 : iex(6)&gt; calls |&gt; Map.get({DateTime, :utc_now, 0}) |&gt; Enum.map(&amp; &amp;1.duration) [35, 14, 12, 12] Replay calls for DateTime.utc_now/0 (use replay/1 with caution in prod!): iex(7)&gt; calls |&gt; Map.get({DateTime, :utc_now, 0}) |&gt; Enum.map(&amp;Replbug.replay/1) [~U[2022-08-31 21:02:02.397846Z], ~U[2022-08-31 21:02:02.397857Z], ~U[2022-08-31 21:02:02.397859Z], ~U[2022-08-31 21:02:02.397860Z]]","ref":"Replbug.html","title":"Replbug","type":"module"},{"doc":"Group the trace by function calls (MFA).","ref":"Replbug.html#calls/1","title":"Replbug.calls/1","type":"function"},{"doc":"","ref":"Replbug.html#get_caller_pids/1","title":"Replbug.get_caller_pids/1","type":"function"},{"doc":"Repeat the call with the same arguments, for instance, after you've applied the changes to the code and reloaded the module. Use replay/1 with caution in prod! Also, it may not work as expected due to changes happened in between the initial call and the time of replay.","ref":"Replbug.html#replay/1","title":"Replbug.replay/1","type":"function"},{"doc":"","ref":"Replbug.html#start/2","title":"Replbug.start/2","type":"function"},{"doc":"","ref":"Replbug.html#stop/0","title":"Replbug.stop/0","type":"function"},{"doc":"","ref":"Replbug.Server.html","title":"Replbug.Server","type":"module"},{"doc":"Utilities on top of Rexbug to facilitate REPL-style debugging in IEx shell.","ref":"readme.html","title":"Replbug","type":"extras"},{"doc":"If available in Hex , the package can be installed by adding replbug to your list of dependencies in mix.exs : def deps do [ { :replbug , &quot;~&gt; 0.1.0&quot; } ] end","ref":"readme.html#installation","title":"Replbug - Installation","type":"extras"},{"doc":"The Erlang Trace BIFs allow to trace Erlang code on live systems. Rexbug displays and/or writes to external files in human-readable format the trace messages emitted by Erlang VM. In many cases, this would be sufficient for the purposes of debugging the code. However, if the size and/or number of tracing messages is large, it becomes more difficult to make sense of what's going on just by visually checking the tracing output. To illustrate the issue, let's add Replbug dependency to our Phoenix server project: defp deps do [ ### Our deps... { :replbug , &quot;~&gt; 0.1.0&quot; } ] end and start a Phoenix server with the LiveDashboard enabled. Now start tracing Phoenix.LiveView.Plug.call/2 in IEx using Rexbug : # Start the tracing for Phoenix.LiveView.Plug.call/2 iex(1)&gt; Rexbug . start ( &quot;Phoenix.LiveView.Plug.call/2 :: return&quot; , time : 60_000 , msgs : 1_000 ) Then hit http://localhost:4000/dashboard or whatever link your LiveDashboard is configured for. We'll see a lot of output in your IEx shell, which is pretty hard to comprehend due to it's size and truncations caused by pretty-printing etc.","ref":"readme.html#motivation","title":"Replbug - Motivation","type":"extras"},{"doc":"Replbug preserves all functionality of Rexbug. Additionally, it allows to materialize trace records as variables that we could inspect and experiment with the collected traces in IEx shell. Let's use Replbug this time to trace the same function: # Make sure to close Rexbug session iex(2)&gt; Rexbug . stop # Start the tracing for Phoenix.LiveView.Plug.call/2 with Replbug iex(3)&gt; Replbug . start ( &quot;Phoenix.LiveView.Plug.call/2 :: return&quot; , time : 60_000 , msgs : 1_000 ) Hit http://localhost:4000/dashboard again. The same amount of output, as with using Rexbug, but now we can get the traces into Iex and inspect them programmatically: iex(3)&gt; traces = Replbug . stop ## `traces` is a map PID -&gt; calls iex(4)&gt; Map . keys ( traces ) [ # PID &lt; 0 . 544 . 0 &gt; , # PID &lt; 0 . 548 . 0 &gt; ] ## These are the processes that made the calls ## Get the list of calls across all processes iex(5)&gt; calls = Replbug . calls ( traces ) ## What calls were traced? iex(6)&gt; Map . keys ( calls ) [ { Phoenix.LiveView.Plug , :call , 2 } ] ## yeah, that&#39;s the one we traced... ## Get the call records (args, returns etc.) iex(7)&gt; call_data = Map . get ( calls , { Phoenix.LiveView.Plug , :call , 2 } ) ## What are the arguments of the first call? iex(8)&gt; [ arg1 , arg2 ] = hd ( call_data ) . args ## We expect arg1 to be a Plug.Conn struct... iex(9)&gt; iex ( 24 ) &gt; is_struct ( arg1 , Plug.Conn ) true ## ...and we are interested in user-agent value... iex(10)&gt; List . keyfind ( arg1 . req_headers , &quot;user-agent&quot; , 0 ) { &quot;user-agent&quot; , &quot;Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/104.0.0.0 Safari/537.36&quot; } ## What are durations of the calls? iex(11)&gt; durations = Enum . map ( calls , &amp; &amp;1 . duration ) [ 9411 , 13954 ] ## Let&#39;s look at the return of the first call. iex ( 12 ) &gt; return = call_data . return ## We expect it to be a Plug.Conn struct as well... iex ( 13 ) &gt; is_struct ( return , Plug.Conn ) true ## Finally, check the HTTP status iex ( 14 ) &gt; return . status 302","ref":"readme.html#solution","title":"Replbug - Solution","type":"extras"},{"doc":"As of now, Replbug only supports tracing of function calls. Support for tracing of inter-process messages is planned for upcoming versions.","ref":"readme.html#status","title":"Replbug - Status","type":"extras"}]